<?php

declare(strict_types=1);

namespace Drupal\navigation_local_tasks;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Menu\LocalTaskManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides base functionality for 'navigation local tasks' block plugins.
 */
abstract class NavigationLocalTasksBase extends BlockBase implements ContainerFactoryPluginInterface {

  protected RouteMatchInterface $routeMatch;
  protected LocalTaskManagerInterface $localTaskManager;
  protected AdminContext $adminContext;

  protected ImmutableConfig $config;

  /**
   * NavigationLocalTasksBase constructor.
   */
  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    RouteMatchInterface $route_match,
    LocalTaskManagerInterface $local_task_manager,
    AdminContext $admin_context,
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->routeMatch = $route_match;
    $this->localTaskManager = $local_task_manager;
    $this->adminContext = $admin_context;
    $this->config = $config_factory->get('navigation_local_tasks.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('plugin.manager.menu.local_task'),
      $container->get('router.admin_context'),
      $container->get('config.factory')
    );
  }

  /**
   * Get local tasks and associated cache data.
   */
  protected function getLocalTasks(): array {
    $cacheability = new CacheableMetadata();
    $cacheability->addCacheableDependency($this->localTaskManager);

    // If the current route belongs to an entity, include cache tags of that
    // entity as well.
    $route_parameters = $this->routeMatch->getParameters()->all();
    foreach ($route_parameters as $parameter) {
      if ($parameter instanceof CacheableDependencyInterface) {
        $cacheability->addCacheableDependency($parameter);
      }
    }

    $links = $this->localTaskManager->getLocalTasks($this->routeMatch->getRouteName(), 0);
    $cacheability = $cacheability->merge($links['cacheability']);
    // Do not display single tabs.
    $tasks = count(Element::getVisibleChildren($links['tabs'])) > 1 ? $links['tabs'] : [];

    // Update key if it does not match route name and remove disallowed tasks.
    $disallowed_tasks = $this->config->get('disallowed_tasks');
    foreach ($tasks as $key => $task) {
      $route_name = $task['#link']['url']->getRouteName();
      if (in_array($route_name, $disallowed_tasks)) {
        unset($tasks[$key]);
        continue;
      }
      if ($key !== $route_name) {
        $tasks[$route_name] = $task;
        unset($tasks[$key]);
      }
    }
    uasort($tasks, 'Drupal\Component\Utility\SortArray::sortByWeightProperty');

    $build = [
      '#local_tasks'=> $tasks,
    ];

    // Add cache meta data.
    $cacheability->applyTo($build);

    return $build;
  }

}
